'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shallowConnectedComponent = exports.mountConnectedComponent = exports.setupJsdom = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createMockStore = createMockStore;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _redux = require('redux');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var setupJsdom = exports.setupJsdom = function setupJsdom() {
  var _require = require('jsdom'),
      JSDOM = _require.JSDOM;

  var jsdom = new JSDOM('<!doctype html><html><body></body></html>');
  var window = jsdom.window;


  function copyProps(src, target) {
    var props = Object.getOwnPropertyNames(src).filter(function (prop) {
      return typeof target[prop] === 'undefined';
    }).reduce(function (result, prop) {
      return _extends({}, result, _defineProperty({}, prop, Object.getOwnPropertyDescriptor(src, prop)));
    }, {});
    Object.defineProperties(target, props);
  }

  global.window = window;
  global.document = window.document;
  global.navigator = {
    userAgent: 'node.js'
  };
  copyProps(window, global);
};

function createMockStore() {
  var reducer = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var preloadedState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;
  var middlewares = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  var mockState = reducer ? reducer(preloadedState, { action: null }) : preloadedState !== undefined ? preloadedState : {};
  var listeners = [];
  var actions = [];

  function mockStore() {
    return {
      getState: function getState() {
        return mockState;
      },
      dispatch: function dispatch(action) {
        if (reducer) {
          mockState = reducer(mockState, action);
        }

        for (var i = 0, l = listeners.length; i < l; i++) {
          listeners[i](mockState);
        }
        actions.push(action);
      },
      subscribe: function subscribe(listener) {
        listeners.push(listener);
        return function () {
          listeners.splice(listeners.indexOf(listener), 1);
        };
      },
      replaceReducer: function replaceReducer(nextReducer) {
        reducer = nextReducer;
      },
      getDispatchedActions: function getDispatchedActions() {
        return actions;
      }
    };
  }

  var MockStoreWithMiddleware = _redux.applyMiddleware.apply(undefined, _toConsumableArray(middlewares))(mockStore);

  return new MockStoreWithMiddleware();
}

var renderingConnectedComponent = function renderingConnectedComponent(mockStore, connectedComponent, props, options, rendering) {
  if (!mockStore) {
    throw new Error('You have not created the store!');
  }

  var componentOptions = { context: { store: mockStore } };
  if (options) {
    componentOptions = _extends({}, options, {
      context: _extends({}, options.context, componentOptions.context)
    });
  }

  var component = rendering === 'mount' ? (0, _enzyme.mount)(_react2.default.createElement(connectedComponent, props), componentOptions) : (0, _enzyme.shallow)(_react2.default.createElement(connectedComponent, props), componentOptions);

  var oldDispatch = mockStore.dispatch;
  mockStore.dispatch = function (action) {
    var result = oldDispatch(action);
    component.update();
    return result;
  };

  return component;
};

var mountConnectedComponent = exports.mountConnectedComponent = function mountConnectedComponent(mockStore, connectedComponent) {
  var props = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
  return renderingConnectedComponent(mockStore, connectedComponent, props, options, 'mount');
};

var shallowConnectedComponent = exports.shallowConnectedComponent = function shallowConnectedComponent(mockStore, connectedComponent) {
  var props = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;
  return renderingConnectedComponent(mockStore, connectedComponent, props, options, 'shallow');
};